// AP(r) Computer Science Marine Biology Simulation:
// The  EnvFileChooser class is copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Component;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.io.File;
import java.util.Set;
import java.util.Iterator;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  An <code>EnvFileChooser</code> is a <code>JFileChooser</code>
 *  subclass that adds
 *  some specialized behavior for opening and saving environment
 *  data files.  The additional features allow for filtering to
 *  just ".dat" files, an alert that confirms before overwriting
 *  an existing file, and an accessory panel on the open dialog
 *  for choosing which bounded/unbounded class to use for
 *  the new environment.
 *
 *  <p>
 *  The <code>EnvFileChooser</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @version 1 August 2002
 **/

public class EnvFileChooser extends JFileChooser
{
    private static final String ENV_EXT = ".dat";
    
    private JPanel openAccessory;
    private JComboBox boundedChooser, unboundedChooser;
    private Class defaultBounded = BoundedEnv.class, defaultUnbounded = UnboundedEnv.class;
    
    
    /** Creates a new EnvFileChooser. The default starting directory will be the "DataFiles"
     *  subdirectory on the current working directory.
     **/
    public EnvFileChooser()
    {
        super(new File(System.getProperty("user.dir") + File.separator + "DataFiles"));
        setFileFilter(new FileFilter() {
            public boolean accept(File f) {
               return (f.getName().toLowerCase().endsWith(ENV_EXT) || f.isDirectory());
            }
            public String getDescription() {
                return("MBS fish data files (*" + ENV_EXT + ")");
        }});
        setFileSelectionMode(JFileChooser.FILES_ONLY);
        makeOpenAccessory();
    }
    
    /** Brings up a modal file chooser dialog allowing the user to choose
     *  the environment file and specify the bounded/unbounded environment
     *  class. 
     *  @param parent the parent of the dialog
     *  @return the return state of the dialog once finished
     **/
    public int showOpenDialog(Component parent)
    {
        setDialogTitle("Open environment file");
        setAccessory(openAccessory);
        rescanCurrentDirectory();
        return super.showOpenDialog(parent);
    }

     
    /** Brings up a modal file chooser dialog allowing the user to save
     *  to an environment file.
     *  @param parent the parent of the dialog
     *  @return the return state of the dialog once finished
    **/
    public int showSaveDialog(Component parent)
    {
        setDialogTitle("Save environment file");
        setAccessory(null);
        rescanCurrentDirectory();
        return super.showSaveDialog(parent);
    }


    /** Called when the user hits the approve button (Save/Open) to 
     *  confirm the selected file is acceptable. Overrides the
     *  JFileChooser to confirm overwrite if choosing a file
     *  that already exists for a save action.
     **/
    public void approveSelection() 
    {
        if (getDialogType() == SAVE_DIALOG)
        {
            File file = getSelectedFile();
            if (!file.getName().endsWith(ENV_EXT))	// add extension if missing
                setSelectedFile(file = new File(file.getAbsolutePath() + ENV_EXT));
            if (file.exists()
                && JOptionPane.showConfirmDialog(this,"File " + file.getName() + 
                            " exists. Overwrite?", "Confirm overwrite",
                             JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION)
                return;
        }
        super.approveSelection(); //if we get here, this will dismiss the dialog
    }
    
    
    /** Returns the bounded environment class selected on the open accessory.
     *  @return the selected bounded environment class
     **/
    public Class boundedClass()
    {
        if (boundedChooser == null)
            return defaultBounded;
        else
            return ((CreateEnvDialog.EnvChoice)boundedChooser.getSelectedItem()).envClass();
    }
    
    /** Returns the unbounded environment class selected on the open accessory.
     *  @return the selected unbounded environment class
     */
   public Class unboundedClass()
    {
        if (unboundedChooser == null)
            return defaultUnbounded;
        else
            return ((CreateEnvDialog.EnvChoice)unboundedChooser.getSelectedItem()).envClass();
    }
    
    
    /** Builds the open accessory with combo boxes for choosing the bounded
     *  and unbounded environment classes.
     **/
   private void makeOpenAccessory()
    {
        openAccessory = new JPanel();
        openAccessory.setLayout(new BoxLayout(openAccessory, BoxLayout.Y_AXIS));
        openAccessory.setBorder(BorderFactory.createEmptyBorder(20, 5, 5, 0));
        Set set = MBSFactory.boundedEnvClasses();
        if (set.size() == 1) // if only one, use that one, don't build combobox
            defaultBounded = (Class)set.iterator().next();
        else if (set.size() > 1) 
        {
            openAccessory.add(new JLabel("Class for bounded:"));
            boundedChooser = new JComboBox();
            CreateEnvDialog.EnvChoice.addChoicesFromFactory(boundedChooser, true);
            boundedChooser.setAlignmentX(Component.LEFT_ALIGNMENT);
            openAccessory.add(boundedChooser);
            openAccessory.add(Box.createRigidArea(new java.awt.Dimension(5, 5)));
        }
        set = MBSFactory.unboundedEnvClasses();
        if (set.size() == 1)
            defaultUnbounded = (Class)set.iterator().next();
        else if (set.size() > 1) 
        {
            openAccessory.add(new JLabel("Class for unbounded:"));
            unboundedChooser = new JComboBox();
            CreateEnvDialog.EnvChoice.addChoicesFromFactory(unboundedChooser, false);
            unboundedChooser.setAlignmentX(Component.LEFT_ALIGNMENT);
            openAccessory.add(unboundedChooser);
        }
    }
    
}
