// AP(r) Computer Science Marine Biology Simulation:
// The  CreateEnvDialog class is copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import java.util.Iterator;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  A <code>CreateEnvDialog</code> is a dialog that allows the user to configure
 *  the specifications for a new environment to be created. It has a
 *  a combo box that lists the various environment classes and a pair of
 *  text fields for the specifying the number of rows and columns for 
 *  a bounded environment.
 *
 *  <p>
 *  The <code>CreateEnvDialog</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @version 1 August 2002
 **/
public class CreateEnvDialog
{    
    private EnvChoice envChoice;
    private int numRows, numCols;
    private JTextField rowField, colField;
    private JComboBox envChooser;
    private JPanel dimPanel;
    private JButton[] optButtons;
    private JDialog dialog;
    
    /** Creates a new CreateEnvDialog and all its controls. 
     *  @param  parent    parent frame for the dialog
     **/
    public CreateEnvDialog(JFrame parent)
    {
        dialog = new JDialog(parent, "Create new environment", true);
        dialog.setContentPane(makeOptionsPanel());
        dialog.pack();
        dialog.setResizable(false);
    }
    
    
    /** Shows the modal dialog that allows the user to create a new
     *  environment.  If the dialog is dismissed by clicking the "OK"
     *  button, a new environment is created to the user's specification 
     *  and returned. If "Cancel" is chosen or there is an error
     *  constructing the environment, <code>null</code> is returned.
     *  @return the newly created Environment or <code>null</code>
     **/
    public Environment showDialog()
    {
        envChoice = null; // clear any previous state
        rowField.requestFocus();
        // center dialog in middle of parent frame
        Component parent = dialog.getParent();
        dialog.setLocation(parent.getX() + parent.getWidth()/2 - dialog.getSize().width/2,
                           parent.getY() + parent.getHeight()/2 - dialog.getSize().height/2);
        dialog.show();   // Modal dialog will block until setVisible(false), see ok/cancel methods
        
        if (envChoice == null) 	// if user cancelled or closed dialog
            return null;	// return null
            
        try 
        {
            if (envChoice.isBounded())
                return MBSFactory.constructEnv(envChoice.envClass(), numRows, numCols);
            else
                return MBSFactory.constructEnv(envChoice.envClass());
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(dialog.getParent(), "Unable to create new environment.\n"
                        + "Reason: " + ex, "Error creating environment", JOptionPane.ERROR_MESSAGE);
            return null;
       }
    }
    
    // callback when user selects new environment class choice from combo box
    private void envChosen()
    {
        // only show the num rows/cols field for bounded environments
        dimPanel.setVisible(((EnvChoice)envChooser.getSelectedItem()).isBounded());
        optButtons[0].requestFocus();
    }
    
    // callback when user clicks cancel button 
    private void cancelClicked()
    {
        envChoice = null;
        dialog.setVisible(false);
    }

   // callback when user clicks ok button 
    private void okClicked()
    {
        try 
        {
            envChoice = (EnvChoice)envChooser.getSelectedItem();
            if (envChoice.isBounded()) 
            {
                rowField.requestFocus();
                numRows = Integer.parseInt(rowField.getText().trim());
                colField.requestFocus();
                numCols = Integer.parseInt(colField.getText().trim());
                if (numRows < 0 || numCols < 0) throw new NumberFormatException();
            }
            dialog.setVisible(false);
        }
        catch (NumberFormatException ex)
        {
            JOptionPane.showMessageDialog(dialog, 
            "Environment dimensions must be positive integers!", 
            "Error creating environment", JOptionPane.ERROR_MESSAGE);
            // we don't dismiss the dialog in this case, leave up for another try
        }
    }

    // Builds the dialog content pane. We have a combo box with
    // environment choices and two text fields for rows/cols that
    // we will show/hide as appropriate.
    private JOptionPane makeOptionsPanel()
    {
        JPanel myControls = new JPanel();
        myControls.setLayout(new BoxLayout(myControls, BoxLayout.Y_AXIS));
        
        JLabel lab;
        myControls.add(lab = new JLabel("Choose environment type: "));
        lab.setAlignmentX(Component.CENTER_ALIGNMENT);
        
        // add all bounded/unbounded environment classes to the combo box
 	    envChooser = new JComboBox();
        EnvChoice.addChoicesFromFactory(envChooser);

        envChooser.setBorder(BorderFactory.createEmptyBorder(5, 0, 10, 0));
   	    envChooser.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) { envChosen(); }});
        myControls.add(envChooser);
        
        dimPanel = new JPanel();
        dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
        dimPanel.add(new JLabel("rows: "));
        dimPanel.add(rowField = new JTextField("10"));
        dimPanel.add(new JLabel("  cols: "));
        dimPanel.add(colField = new JTextField("10"));
        myControls.add(dimPanel);
        
        JOptionPane optPane = new JOptionPane(myControls, JOptionPane.QUESTION_MESSAGE);
        optButtons = new JButton[] { new JButton("Create"), new JButton("Cancel")};
        optPane.setOptions(optButtons);
        optPane.setInitialValue(optButtons[0]);
        optButtons[1].addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { cancelClicked(); }});
        optButtons[0].addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) { okClicked(); }});
        return optPane;
    }


    /** Nested class  used to hold the per-item information 
     *  for the entries in the combo box of environment
     *  choices. Each item represents an environment
     *  class and tracks whether this is a bounded or unbounded type.
     */
    static class EnvChoice	
    {
        private Class cls;
        private boolean isBounded;
        
        public EnvChoice(Class cl, boolean isB) { cls = cl; isBounded = isB; }
        public boolean isBounded() { return isBounded; }
        public Class envClass() { return cls; }
        public String toString() { return cls.getName(); }
        
        public static void addChoicesFromFactory(JComboBox cb)
        {
            addChoicesFromFactory(cb, true);
            addChoicesFromFactory(cb, false);
        }
        
        public static void addChoicesFromFactory(JComboBox cb, boolean isBounded)
        {
            Iterator iter = isBounded ? MBSFactory.boundedEnvClasses().iterator() : 
                                        MBSFactory.unboundedEnvClasses().iterator();
            while (iter.hasNext())
                cb.addItem(new EnvChoice((Class)iter.next(), isBounded));
        }
    }

}

